'
' This file contains an Implementation of a Software Mode 0 Master SPI interface.
'
REMINDERS = OFF 
'-----------------------------------------------------------------------------
' transmits one byte <WREG> using the SPI protocol mode 0
' SCK is idle-low, and bits are latched on SCK rising.
		
SPI_OUT0	MACRO BYTEIN	
        #IF(PRM_COUNT >= 1)
        	#IF((PRM_1 == NUM8) || (PRM_1 == NUM16) || (PRM_1 == NUM32))
                Num_Wreg BYTEIN
        	#ELSE
@        		Movf BYTEIN,W
        	#ENDIF
        #ELSE
        	movlw 0 
        #ENDIF 
        GoSub _SPI_OUT_MODE_0
		ENDM 
        
#IFDEF SPI_OUT0#REQ
'
' Input		: WREG = 8-bit value to send
' 			: MMC_CLK is the PORT.BIT of the SPI Clock pin
'			: MMC_SDI is the PORT.BIT of the SPI Data IN Pin
'			: MMC_SDO is the PORT.BIT of the SPI Data OUT Pin
' Output	: PRODL = 8-bit value received
'
_SPI_OUT_MODE_0:      
	movwf PRODL 							' Transfer WREG into PRODL
	PRODH = 8								' Single byte SPI loop
SPI_OUT_MODE0_LP:
		MMC_SDO = PRODL & $80 				' Put current outgoing bit on MMC_SDO
		PRODL = PRODL << 1 					' Shift next bit into MSB
		Set MMC_CLK							' Set MMC_CLK high
		PRODL = PRODL | MMC_SDI 			' Capture current bit on MMC_SDI
   		Clear MMC_CLK 						' Pull MMC_CLK low
 	djnz PRODH,SPI_OUT_MODE0_LP				' Do 8-bits
	Return	
#ENDIF
'-----------------------------------------------------------------------------
' Receives one byte <WREG> using the SPI protocol mode 0 
' SCK is idle-low, and bits are latched on SCK rising.
' 
' If a parameter is present, the amount of bytes received is incremented in SECTOR_BYTE_COUNTER
'		
SPI_IN0	MACRO PARAMETER_1       
        #IF(PRM_COUNT == 1)
        	GoSub _SPI_IN_MODE_0_COUNT
        #ELSE
        	GoSub _SPI_IN_MODE_0
        #ENDIF
        #IF(SPI_IN0_RETURN == 1)
        	#IF(RETURN_TYPE == Byte)
            	#IF((RETURN_VAR != WREG) && (RETURN_VAR != PRODL))
            	Wreg_Byte RETURN_VAR
				#ENDIF
            #ENDIF
            #IF(RETURN_TYPE == Word)
            	Wreg_Byte RETURN_VAR
                #IF(PRM_COUNT == 1)
        			GoSub _SPI_IN_MODE_0_COUNT
        		#ELSE
        			GoSub _SPI_IN_MODE_0
        		#ENDIF
                Wreg_Byte RETURN_VAR + 1
            #ENDIF
            #IF(RETURN_TYPE == DWord)
            	Wreg_Byte RETURN_VAR
                #IF(PRM_COUNT == 1)
        			GoSub _SPI_IN_MODE_0_COUNT
        		#ELSE
        			GoSub _SPI_IN_MODE_0
        		#ENDIF
                Wreg_Byte RETURN_VAR + 1
                #IF(PRM_COUNT == 1)
        			GoSub _SPI_IN_MODE_0_COUNT
        		#ELSE
        			GoSub _SPI_IN_MODE_0
        		#ENDIF
                Wreg_Byte RETURN_VAR + 2
                #IF(PRM_COUNT == 1)
        			GoSub _SPI_IN_MODE_0_COUNT
        		#ELSE
        			GoSub _SPI_IN_MODE_0
        		#ENDIF
                Wreg_Byte RETURN_VAR + 3
            #ENDIF
        #ENDIF
		ENDM 
                
#IFDEF SPI_IN0#REQ
'
' Input		: MMC_CLK is the PORT.BIT of the SPI Clock pin
'			: MMC_SDI is the PORT.BIT of the SPI Data IN Pin
'			: MMC_SDO is the PORT.BIT of the SPI Data OUT Pin
' Output	: PRODL = 8-bit value received
'
_SPI_IN_MODE_0_COUNT:
	Inc SECTOR_BYTE_COUNTER					' Increment the byte counter
_SPI_IN_MODE_0:
	PRODH = 8								' Single byte SPI loop
SPI_IN_MODE0_LP:
		PRODL = PRODL << 1 					' Shift next bit into MSB
		Set MMC_CLK							' Set MMC_CLK high
		PRODL = PRODL | MMC_SDI 			' Capture current bit on MMC_SDI
		Clear MMC_CLK 						' Pull MMC_CLK low
	djnz PRODH,SPI_IN_MODE0_LP				' Do 8-bits
 	movf PRODL,W							' Transfer PRODL into WREG
	Return	
#ENDIF
REMINDERS = On
